{ TSortGrid 1.30 Copyright 1998 by Karsten Bendsen

 Please read sortgrid.txt, as there is much information about TSortGrid

 Contents:

 Misc NonMember Functions: Create and Destroy methods
 Misc OnEvent:             Misc Events that I couldn't fit under another section
 Proportional Scrollbar:   Proportional ScrollBars, DOSN'T work 100%, please read section for more information.
 Cell Format:              Custom look for each cell
 MultiSelect:              Multiselect with [Ctrl] and [Shift], Works 99%
 TableEdit procedures:     All the procedures missing in TStringGrid, row and col functions,
 Sort:                     Sort procedures
 Search & Filter:          All procedure for search and filter the grid
 File:                     Load & Save
 Junk:                     Misc procedures that dosn't work or are incomplete, perhaps you could have a look at it.
}
unit SortGrid;

interface

uses
  ExtCtrls,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, Grids, Printers;

type
  TPrintMode = (pmPrint, pmPreview, pmPageCount);
  TSortStyle = (ssAutomatic, ssNormal, ssNumeric, ssExtended, ssDateTime, ssTime);
  TSortDirection = (sdAscending, sdDescending);

  TSortOptions = class(TPersistent)
  private
    fSortStyle: TSortStyle;
    fSortCaseSen: Boolean;
    fSortCol: Integer;
    fSortDirection: TSortDirection;
  protected
  public
    constructor Create;
    destructor Destroy; override;
  private
  published
    property SortStyle: TSortStyle read fSortStyle write fSortStyle;
    property SortCaseSensitive: Boolean read fSortCaseSen write fSortCaseSen;
    property SortCol: Integer read fSortCol;
    property SortDirection: TSortDirection read fSortDirection write fSortDirection;
  end;

  TPrintOptions = class(TPersistent)
  private
    fJobTitle: String;
    fPageTitle: String;
    fCopies: Cardinal;
    fPreviewPage,fFromRow, fToRow: Cardinal;
  protected
  public
  private
  published
    property JobTitle: string read fJobTitle write fJobTitle;
    property PageTitle: string read fPageTitle write fPageTitle;
    property Copies: Cardinal read fCopies write fCopies default 1;
    property FromRow: Cardinal read fFromRow write fFromRow;
    property ToRow: Cardinal read fToRow write fToRow;
    property PreviewPage: Cardinal read fPreviewPage write fPreviewPage;
  end;

  TSortedListEntry = record
    Str: String;
    RowNum: Longint;
  end;
  PSortedListEntry = ^TSortedListEntry;
  TSortedList = class(TList)
  public
    function GetItem(const I: Integer): PSortedListEntry;
    procedure Reset;
  end;

  TCellBevelStyle = (cbNone, cbRaised, cbLowered);
  {*** NOTE: This is one of the structures in TFormatOptions.}
  TCellBevel = record
      Style: TCellBevelStyle;
      UpperLeftColor: TColor;
      LowerRightColor: TColor;
  end;
  TVertAlignment = (taTopJustify, taBottomJustify, taMiddle);
  {*** NOTE: These are the display options you can set for each cell in OnGetCellFormat.}
  TFormatOptions = record
      Brush: TBrush;
      Font: TFont;
      AlignmentHorz: TAlignment;
      AlignmentVert: TVertAlignment;
      Bevel: TCellBevel;
      HideText: Boolean;
  end;

  TSearchOptions = class;

  TFormatDrawCellEvent = procedure(Sender: TObject; Col, Row: Longint; State: TGridDrawState; var FormatOptions: TFormatOptions) of object;
  TUpdateGridEvent = procedure(Sender: TObject; Index: Longint) of object;
  TSizeChangedEvent = procedure(Sender: TObject; OldColCount, OldRowCount: Longint) of object;

  TBeginSortEvent = procedure(Sender: TObject; Col: Longint) of object;
  TEndSortEvent = procedure(Sender: TObject; Col: Longint) of object;

  {Here's the main new class: TSortGrid}
  TSortGrid = class(TStringGrid)
  private
    { Private declarations }
    BitmapUp,BitmapDown: TBitmap;
    FLeftCol: Integer;
    fSearchOptions: TSearchOptions;
    fSortOptions: TSortOptions;
    fFixedRows: Integer;
    fPrintOptions: TPrintOptions;
    fLastRow: Integer;
    fMouseDownRow: Integer;
    fMouseDownCol: Integer;
    fMultiSelect:Boolean;
    fOriginRowCount:integer;
    fSelectRows:TStrings;
    fUpdateOnSizeChange: Boolean;
    fSecondaryIndex: String;

    fSortedList: TSortedList;
    fAlignmentHorz: TAlignment;
    fAlignmentVert: TVertAlignment;
    fClickSorting: Boolean;
    fBevelStyle: TCellBevelStyle;
    fProportionalScrollBars: Boolean;
    fExtendedKeys: Boolean;
    fSorting: Boolean;
    fModified: Boolean;
    fEntered: Boolean;
    fOnGetCellFormat: TFormatDrawCellEvent;
    fOnRowInsert: TUpdateGridEvent;
    fOnRowDelete: TUpdateGridEvent;
    fOnColumnInsert: TUpdateGridEvent;
    fOnColumnDelete: TUpdateGridEvent;
    fOnColumnWidthsChanged: TNotifyEvent;
    fOnRowHeightsChanged: TNotifyEvent;
    fOnSizeChanged: TSizeChangedEvent;
    fOnBeginSort: TBeginSortEvent;
    fOnEndSort: TEndSortEvent;
    fPageCount: Cardinal;

    procedure SetAlignmentHorz(Value: TAlignment);
    procedure SetAlignmentVert(Value: TVertAlignment);
    procedure SetBevelStyle(Value: TCellBevelStyle);
    procedure WM_Size(var Msg: TWMSize); message WM_SIZE;
    procedure SetProportionalScrollBars(Value: Boolean);
    function Search(StartRow: Cardinal): Boolean;
    procedure DeSelectAll;
    procedure SetMultiSelect(value:Boolean);
    procedure DrawToCanvas(ACanvas: TCanvas; Mode: TPrintMode; FromRow, ToRow: Integer);
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure ColumnMoved(FromIndex, ToIndex: Integer); override;
    procedure SetSearchOptions(value: TSearchOptions);
    procedure SetSortOptions(value: TSortOptions);
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;

    function DetermineSortStyle(const ACol: Longint): TSortStyle; virtual;
    procedure InitializeFormatOptions(ACol,ARow: Integer; var FmtOpts: TFormatOptions);
    procedure ColWidthsChanged; override;
    procedure RowHeightsChanged; override;
    procedure SizeChanged(OldColCount, OldRowCount: Longint); override;

    procedure UpdateScrollPage; virtual;
    procedure SetEditText(ACol, ARow: Longint; const Value: string); override;
public
    { Public declarations }
    { run time properties }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function LoadFromFile(const FileName: String; LoadWidth: Boolean): Boolean;
    procedure SaveToFile(const FileName: String; SaveWidth: Boolean);

    function FindFirst: Boolean;
    function FindNext: Boolean;

    function GetSelectRow(ARow: Integer):Boolean;
    function GetSelectRowCount: Integer;
    procedure SetSelectRow(ARow: Integer; Selected: Boolean);

    procedure SortColumn(ACol: LongInt); virtual;
    property Sorting: Boolean read fSorting default False;
    property Modified: Boolean read fModified write fModified default False;

    procedure Clear; virtual;
    procedure BeginUpdate; Virtual;
    procedure EndUpdate; Virtual;
    procedure MoveTo(ACol, ARow: Longint); virtual;
    procedure InsertRow(ARow: Longint); virtual;
    procedure InsertColumn(ACol: Longint); virtual;
    procedure DeleteRow(ARow: Longint); virtual;
    procedure DeleteColumn(ACol: Longint); virtual;
    procedure MoveRow(FromIndex, ToIndex: Longint); virtual;
    procedure MoveColumn(FromIndex, ToIndex: Longint); virtual;
    procedure SwapRows(ARow1, ARow2: Longint); virtual;
    procedure SwapColumns(ACol1, ACol2: Longint); virtual;
    procedure AutoSizeCol(const ACol: LongInt); virtual;
    procedure AutoSizeColumns; virtual;

    function GetCellDrawState(const ACol, ARow: Longint): TGridDrawState;

    procedure Print;
    procedure PrintPreview(Image: TImage);
    function PageCount: Integer;
  published
    { Published declarations }
    property SearchOptions: TSearchOptions read fSearchOptions write SetSearchOptions;
    property SortOptions: TSortOptions read fSortOptions write SetSortOptions;
    property FixedRows: Integer read fFixedRows default 1;
    property PrintOptions: TPrintOptions read fPrintOptions write fPrintOptions;
    property MultiSelect: Boolean read fMultiSelect write SetMultiSelect default false;
    property AlignmentHorz: TAlignment read fAlignmentHorz write SetAlignmentHorz;
    property AlignmentVert: TVertAlignment read fAlignmentVert write SetAlignmentVert;
    property BevelStyle: TCellBevelStyle read fBevelStyle write SetBevelStyle;
    property ProportionalScrollBars: Boolean read fProportionalScrollBars write SetProportionalScrollBars;
    property ExtendedKeys: Boolean read fExtendedKeys write fExtendedKeys;
    property UpdateOnSizeChange: Boolean read fUpdateOnSizeChange write fUpdateOnSizeChange;
    property SecondaryIndex: String read fSecondaryIndex write fSecondaryIndex;

    { Published events }
    property OnGetCellFormat: TFormatDrawCellEvent read fOnGetCellFormat write fOnGetCellFormat;
    property OnRowInsert: TUpdateGridEvent read fOnRowInsert write fOnRowInsert;
    property OnRowDelete: TUpdateGridEvent read fOnRowDelete write fOnRowDelete;
    property OnColumnInsert: TUpdateGridEvent read fOnColumnInsert write fOnColumnInsert;
    property OnColumnDelete: TUpdateGridEvent read fOnColumnDelete write fOnColumnDelete;
    property OnColumnWidthsChanged: TNotifyEvent read fOnColumnWidthsChanged write fOnColumnWidthsChanged;
    property OnRowHeightsChanged: TNotifyEvent read fOnRowHeightsChanged write fOnRowHeightsChanged;
    property OnSizeChanged: TSizeChangedEvent read fOnSizeChanged write fOnSizeChanged;

    property OnBeginSort: TBeginSortEvent read fOnBeginSort write fOnBeginSort;
    property OnEndSort: TEndSortEvent read fOnEndSort write fOnEndSort;
  end;

  TSearchOptions = class(TPersistent)
  private
    fGrid: TSortGrid;
    fFiltered: Boolean;
    fSearchCol: Integer;
    fSearchText: String;
    fCaseSensitive: Boolean;
  protected
    procedure SetGrid(Grid: TSortGrid );
    procedure SetFiltered(Value: Boolean);
    property Grid: TSortGrid read fGrid write SetGrid;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property Filtered: Boolean read fFiltered write SetFiltered;
    property SearchCol: Integer read fSearchCol write fSearchCol default -1;
    property CaseSensitive: Boolean read fCaseSensitive write fCaseSensitive;
    property SearchText: String read fSearchText write fSearchText;
  end;

  procedure Register;
  function ExtendedCompare(const Str1, Str2: String): Integer;
  function NormalCompare(const Str1, Str2: String): Integer;
  function DateTimeCompare(const Str1, Str2: String): Integer;
  function NumericCompare(const Str1, Str2: String): Integer;
  function TimeCompare(const Str1, Str2: String): Integer;

implementation

{$R SortGrid.Res}

var
  VarSortOptions: TSortOptions;
  FilterCount: Integer = 0;
  OldUpdateSize: Boolean;

{*****Misc Non-Member Functions************************************************}

procedure Register;
begin
  RegisterComponents('Samples', [TSortGrid]);
end;

constructor TSearchOptions.Create;
begin
  Filtered := False;
  fSearchCol := -1;
  fSearchText := '';
  fCaseSensitive := False;
end;

destructor TSearchOptions.Destroy;
begin
  inherited Destroy;
end;

constructor TSortOptions.Create;
begin
  fSortDirection := sdAscending;
  fSortCol := 0;
  fSortStyle := ssExtended;
  fSortCaseSen := False;
end;

destructor TSortOptions.Destroy;
begin
  inherited Destroy;
end;

constructor TSortGrid.Create(AOwner: TComponent);
var
//  FmtOpts: TFormatOptions;
  I: Integer;
begin
  inherited Create(AOwner);
  fSortedList:=TSortedList.Create;
  fAlignmentHorz:=taLeftJustify;
  fAlignmentVert:=taTopJustify;
  fClickSorting:=True;
  fBevelStyle:=cbNone;
  fProportionalScrollBars:=True;
  fExtendedKeys:=False;
  fSorting:=True;
  fModified:=False;
  fMouseDownCol := -1;
  fEntered:=False;
  FixedCols:=0;

  fOriginRowCount:=RowCount;
  fmousedownrow := -1;
  fSelectRows:=TStringList.Create;
  for I := 0 to RowCount-1 do fSelectRows.Add('N');
  fLastRow:=1;
  fSelectRows[1] := 'Y';

  fSearchOptions := TSearchOptions.Create;
  fSearchOptions.Grid := Self;

  fSortOptions := TSortOptions.Create;
  fPrintOptions := TPrintOptions.Create;

  BitMapUp := TBitMap.Create;
  BitMapDown := TBitMap.Create;
  BitMapUp.LoadFromResourceName(HInstance,'UP');
  BitMapDown.LoadFromResourceName(HInstance,'DOWN');
end;

destructor TSortGrid.Destroy;
begin
  fSortedList.Reset;
  fSortedList.Free;
  fSelectRows.Free;
  BitmapUp.Free;
  BitmapDown.Free;
  inherited Destroy;
end;

{******Misc OnEvent************************************************************}

procedure TSortGrid.KeyDown(var Key: Word; Shift: TShiftState);
var
  I: Integer;
begin
//Extended Keys
  if ExtendedKeys and not EditorMode then
  begin
    if Shift = [ssCtrl] then
    begin
      case Key of
        VK_INSERT: InsertRow(Row);
        VK_DELETE: if RowCount > 2 then DeleteRow(Row);
      end;
    end
    else if Shift = [ssCtrl, ssShift] then
    begin
      case Key of
        VK_INSERT: InsertColumn(Col);
        VK_DELETE: if ColCount > 1 then DeleteColumn(Col);
      end;
    end;
  end;

//MultiSelect
  if (fMultiSelect = False) or (Shift = []) then
    if (Key=VK_HOME) or (Key=VK_END) or (Key=VK_PRIOR) or (Key=VK_NEXT)
      or (Key=VK_UP) or (Key=VK_DOWN) or (Key=VK_LEFT) or (Key=VK_RIGHT) then
        DeSelectAll;

  if (MultiSelect = True) and (Shift = [ssShift]) then
  begin
    if (Key=VK_HOME) or (Key=VK_END) or (Key=VK_PRIOR) or (Key=VK_NEXT)
      or (Key=VK_UP) or (Key=VK_DOWN) or (Key=VK_LEFT) or (Key=VK_RIGHT)then
      DeselectAll;
      if Row > fLastRow then
        for I := fLastRow to Row do fSelectRows[I] := 'Y'
      else
        for I := fLastRow downto Row do fSelectRows[I] := 'Y';
      fLastRow:=Row;
  end;
 
//  Invalidate;
  inherited KeyDown(Key, Shift);
end;

procedure TSortGrid.ColumnMoved(FromIndex, ToIndex: Integer);
begin
  if SortOptions.fSortCol = FromIndex then
    SortColumn(ToIndex);
  if (FromIndex > SortOptions.fSortCol) and (ToIndex <= SortOptions.fSortCol) then
    SortColumn(SortOptions.fSortCol+1);
  if (FromIndex < SortOptions.fSortCol) and (ToIndex >= SortOptions.fSortCol) then
    SortColumn(SortOptions.fSortCol-1);
  inherited ColumnMoved(FromIndex,ToIndex);
end;

procedure TSortGrid.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ARow,ACol : longint;
  I: integer;
begin
  //SetLeftcol
  LeftCol := FLeftCol;

  MouseToCell(x,Y,ACol,ARow);
  //SortClick
  if (fgridstate <> gsColSizing) and (ARow = 0) and (ACol = fMouseDownCol) then
  begin
    if (SortOptions.SortDirection = sdAscending) and (SortOptions.fSortCol = ACol) then
      SortOptions.fSortDirection := sdDescending
    else
      SortOptions.fSortDirection := sdAscending;
    if Button = mbRight then
      SortOptions.fSortDirection := sdDescending;

    SortColumn(ACol);
  end;

// MultiSelect
  if (fMultiSelect = True) and (ARow <> fMouseDownRow) then
    if ARow > fMouseDownRow then
      for I := fMouseDownRow to ARow do
        fSelectRows[I] := 'Y'
    else
    try  //Bug fix by Dipesh
    except
      for I := fMouseDownRow downto ARow do
        FSelectRows[I] := 'Y';
    end;

  if (fMultiSelect = True) and (Shift = [ssShift]) then
  begin
    DeSelectAll;
    if ARow > fLastRow then
      for I := fLastRow to ARow do
        fSelectRows[I] := 'Y'
    else
      for I := fLastRow downto ARow do
        fSelectRows[I] := 'Y';
  end;

//  Invalidate;
  inherited MouseUp(Button,Shift,X,Y);
end;

{*******Proportional Scrollbar*************************************************}

procedure TSortGrid.SetProportionalScrollBars(Value: Boolean);
begin
  fProportionalScrollBars:=Value;
  UpdateScrollPage;
end;

procedure TSortGrid.ColWidthsChanged;
begin
  inherited ColWidthsChanged;
  if Assigned(fOnColumnWidthsChanged) then
     fOnColumnWidthsChanged(Self);
  UpdateScrollPage;
end;

procedure TSortGrid.RowHeightsChanged;
begin
     inherited RowHeightsChanged;
     if Assigned(fOnRowHeightsChanged) then
        fOnRowHeightsChanged(Self);
     UpdateScrollPage;
end;

procedure TSortGrid.WM_Size(var Msg: TWMSize);
//var
//  iCol, iWidth: Integer;
begin
  inherited;

  //Stretch LastCol
{  if ColCount = (LeftCol+VisibleColCount) then
  begin
    iWidth := 0;
    for iCol := LeftCol to ColCount-2 do
      Inc(iWidth, ColWidths[Col]);
   ColWidths[ColCount-1] := Width-iWidth+50;
  end;}

  Invalidate;  //Avoid some gfx mistakes, but the grid flickers :-(
  UpdateScrollPage;
end;

procedure TSortGrid.UpdateScrollPage;
    function LMax(const A, B: Longint): Longint;
    begin
         Result:=B;
         if A > B then Result:=A;
    end;
var
  SI: TScrollInfo;
begin
     {Make the scroll bar(s) proportional.}
     {To do this correctly, I should sum colwidths and rowheights,
     but I just approximate by basing the proportion on visible rows or cols
     divided by row or col count...}
     {Also, I can't really figure out Borland's scroll bar range and position
     scheme.  Thus, sometimes when you click on the end of the scroll bar, you
     still have to scroll farther with the arrows to actually get to the end
     of the grid.  If anyone knows how to fix this, PLEASE let me know...}
  if (ScrollBars = ssVertical) or (ScrollBars = ssBoth) then
  begin
    SI.cbSize:=sizeof(SI);
    SI.fMask:=SIF_PAGE or SIF_POS or SIF_RANGE;
    GetScrollInfo(Handle, SB_VERT, SI);
    SI.fMask:=SIF_PAGE;
    if ProportionalScrollBars then
      SI.nPage:=LMax(Round(((1.0*(VisibleRowCount+1))/RowCount)*(SI.nMax-SI.nMin+1)), 1)
    else
      SI.nPage:=0;
    SetScrollInfo(Handle, SB_VERT, SI, false);
  end;

  if (ScrollBars = ssHorizontal) or (ScrollBars = ssBoth) then
  begin
    SI.cbSize:=sizeof(SI);
    SI.fMask:=SIF_PAGE or SIF_POS or SIF_RANGE;
    GetScrollInfo(Handle, SB_HORZ, SI);
    SI.fMask:=SIF_PAGE;
    if ProportionalScrollBars then
      SI.nPage:=LMax(Round(((1.0*(VisibleColCount)/ColCount)*(SI.nMax-SI.nMin+1))), 1)
    else
      SI.nPage:=0;
    SetScrollInfo(Handle, SB_HORZ, SI, True);
  end;
end;






{********Cell Format***********************************************************}

procedure TSortGrid.SetAlignmentHorz(Value: TAlignment);
begin
  fAlignmentHorz:=Value;
  Invalidate;
end;

procedure TSortGrid.SetAlignmentVert(Value: TVertAlignment);
begin
  fAlignmentVert:=Value;
  Invalidate;
end;

procedure TSortGrid.SetBevelStyle(Value: TCellBevelStyle);
begin
  fBevelStyle:=Value;
  Invalidate;
end;

function TSortGrid.GetCellDrawState(const ACol, ARow: Longint): TGridDrawState;
    function PointInGridRect(Col, Row: Longint; const Rect: TGridRect): Boolean;
    begin
      Result := (Col >= Rect.Left) and (Col <= Rect.Right) and (Row >= Rect.Top)
        and (Row <= Rect.Bottom);
    end;
var
   DrawState: TGridDrawState;
begin
     DrawState:=[];
     if (ARow < 1) and (ACol < 0) then Include(DrawState, gdFixed);
     if Focused and (ARow = Row) and (ACol = Col) then Include(DrawState, gdFocused);
     if PointInGridRect(ACol, ACol, Selection) then Include(DrawState, gdSelected);
     Result:=DrawState;
end;

procedure TSortGrid.InitializeFormatOptions(ACol,ARow: Integer; var FmtOpts: TFormatOptions);
begin
  //Setup good defaults for FormatOptions.
  FmtOpts.HideText:=False;
  FmtOpts.Font := Canvas.Font;
  FmtOpts.Brush := Canvas.Brush;
  FmtOpts.AlignmentHorz := AlignmentHorz;
  FmtOpts.AlignmentVert := AlignmentVert;
  FmtOpts.Bevel.Style := BevelStyle;
  //Set defaults for the bevel colors.
  case BevelStyle of
    cbRaised:
      begin
        FmtOpts.Bevel.UpperLeftColor:=clBtnHighlight;
        FmtOpts.Bevel.LowerRightColor:=clBtnShadow;
      end;
    cbLowered:
      begin
        FmtOpts.Bevel.UpperLeftColor:=clBtnShadow;
        FmtOpts.Bevel.LowerRightColor:=clBtnHighlight;
      end;
    else
      FmtOpts.Bevel.UpperLeftColor:=clWindow;
    FmtOpts.Bevel.LowerRightColor:=clWindow;
  end;
  //Now do the OnGetCellFormat event if necessary.
  if Assigned(fOnGetCellFormat) then
    fOnGetCellFormat(Self, ACol, ARow, GetCellDrawState(ACol,ARow), FmtOpts);
end;

procedure TSortGrid.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
var
  xOffset, yOffset: Integer;
  FmtOpts: TFormatOptions;

  Text: String;
  Width: Integer;
  I: Integer;
begin
  //MultiSelect
  if MultiSelect = True then
  begin
    if fOriginRowCount > RowCount then
    begin
      fSelectRows.Clear;
      for I := 0 to RowCount - 1 do fSelectRows.Add('N');
      fOriginRowCount := RowCount;
    end;

    if fOriginRowCount < RowCount then
    begin
      for I := FOriginRowCount to RowCount - 1 do fSelectRows.Add('N');
      FOriginRowCount := RowCount;
    end;

    if (fSelectRows[ARow] = 'Y') and (ACol >= 0) and (ARow >= 1) then
    begin
      if GetSelectRow(Row) = False then
      begin
        Row := ARow;
        fLastRow := ARow;
      end;
      Canvas.Font.Color := clHighlightText;
      Canvas.Brush.Color := clHighlight;
      Canvas.FillRect(ARect);
    end;
  end;

  InitializeFormatOptions(ACol,ARow, FmtOpts);//AState
  Canvas.Font := FmtOpts.Font;

  if DefaultDrawing then
  begin
    Text := Cells[ACol, ARow];
    //Text displayed in Header
    if ARow = 0 then
    begin
      Width := ColWidths[ACol]-1;
      if SortOptions.fSortCol = ACol then
        Dec(Width,11);//width of up/down bitmap
      if Canvas.TextWidth(Text) > Width then
      begin
        Text[Length(Text)-1] := '.';
        Text[Length(Text)] := '.';
        while (Canvas.TextWidth(Text) > Width) and (Length(Text) > 3) do
        begin
          Delete(Text, Length(Text), 1);
          Text[Length(Text)-1] := '.';
        end;
      end;
    end;

    //Calculate horizontal offset.
    case FmtOpts.AlignmentHorz of
      taRightJustify: xOffset := ARect.Right-ARect.Left-Canvas.TextWidth( Text )-2;
      taCenter: xOffset := (ARect.Right-ARect.Left-Canvas.TextWidth( Text )) div 2;
      else
        xOffset:=2;
    end;

    //Calculate vertical offset.
    case FmtOpts.AlignmentVert of
      taBottomJustify: yOffset:=ARect.Bottom-ARect.Top-Canvas.TextHeight( Text )-3;
      taMiddle: yOffset:=(ARect.Bottom-ARect.Top-Canvas.TextHeight( Text )) div 2;
      else
        yOffset:=2;
    end;

    //Draw Up/Down Bitmap
    if (SortOptions.fSortCol = ACol) then
    begin
      if ARow = 0 then
        Inc(xOffset,11); //width of up/down bitmap
      //Width ~ a X-OffSet value, so Up/Down Bitmap fits, when HeaderWidth less that Bitmap.width
      if ARect.Right-ARect.Left > 10 then Width := 8
      else Width := ARect.Right-ARect.Left-3;
      //I ~ an Y-Offset value for Up/Down Bitmap
      I := (RowHeights[0] div 2) -4;

      if SortOptions.SortDirection = sdAscending then
        Canvas.CopyRect(Rect(ARect.Left+3,I,ARect.Left+Width+3,I+7), BitmapUp.Canvas, Rect(0,0,Width,7))
      else
        Canvas.CopyRect(Rect(ARect.Left+3,I,ARect.Left+Width+3,I+7), BitmapDown.Canvas, Rect(0,0,Width,7));
    end;

    //Now do the text drawing.
    if not FmtOpts.HideText then
      Canvas.TextRect(ARect, ARect.Left+xOffset, ARect.Top+yOffset, Text)
    else
      Canvas.TextRect(ARect, ARect.Left+xOffset, ARect.Top+yOffset, '');
    //Draw Bevel.
    if (FmtOpts.Bevel.Style <> cbNone) and
       (ACol >= 0) and (ARow >= 1) then
    begin
      //Draw from bottom-most lines out to mimic behaviour of
      //fixed cells when FixedXXXXLine is toggled.
      with ARect do
      begin
        if goFixedVertLine in Options then
        begin
          Canvas.Pen.Color := FmtOpts.Bevel.LowerRightColor;
          Canvas.PolyLine([Point(Right-1, Top), Point(Right-1, Bottom)]);
        end;
        if goFixedHorzLine in Options then
        begin
          Canvas.Pen.Color := FmtOpts.Bevel.LowerRightColor;
          Canvas.PolyLine([Point(Left, Bottom-1), Point(Right, Bottom-1)]);
        end;
        if goFixedVertLine in Options then
        begin
          Canvas.Pen.Color := FmtOpts.Bevel.UpperLeftColor;
          Canvas.PolyLine([Point(Left, Top), Point(Left, Bottom)]);
        end;
        if goFixedHorzLine in Options then
        begin
          Canvas.Pen.Color := FmtOpts.Bevel.UpperLeftColor;
          Canvas.PolyLine([Point(Left, Top), Point(Right, Top)]);
        end;
      end;
    end;

    if Assigned( OnDrawCell ) then
      OnDrawCell(Self, ACol, ARow, ARect, AState );
  end
  else
    inherited DrawCell(ACol, ARow, ARect, AState);
end;




{******MultiSelect*************************************************************}

procedure TSortGrid.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if (fMultiSelect = True) and (Shift = []) then
  begin
    if (Key=VK_HOME) or (Key=VK_END) or (Key=VK_PRIOR) or (Key=VK_NEXT)
      or (Key=VK_UP) or (Key=VK_DOWN) or (Key=VK_LEFT) or (Key=VK_RIGHT)then
    begin
      fSelectRows[Row] := 'Y';
      fLastRow := Row;
    end;
  end;
end;

procedure TSortGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ARow, ACol: longint;
begin
  //Set the leftcol, changes when selecting rows besides the visible rows with multiselect
  FLeftCol := LeftCol;

  MouseToCell(x,y,Acol,Arow);
  if Arow < 0 then
    Arow := RowCount-1;
  fMouseDownRow := Arow;

  fMouseDownCol := ACol;//If col changes betweem mouse Up&Down, don't sort, bacause col has been moved

  if (fMultiSelect = True) and (Shift = [ssLeft]) then
  begin
    DeSelectAll;
    fSelectRows[ARow] := 'Y';
    fLastRow := ARow;
  end;

  if (fMultiSelect = True) and (Shift = [ssLeft,ssCtrl]) then
  begin
    if fSelectRows[ARow] = 'Y' then
      fSelectRows[ARow] := 'N'
    else
    begin
      fLastRow := ARow;
      fSelectRows[ARow] := 'Y';
    end;
  end;

//  Invalidate;
  inherited MouseDown(Button,Shift,X,Y);
end;

function TSortGrid.GetSelectRow(ARow: Integer):Boolean;
begin
  if fSelectRows[ARow]='Y' then
    Result := True
  else
    Result := False;
end;

procedure TSortGrid.SetSelectRow(ARow: Integer; Selected: Boolean);
begin
  if Selected then
    fSelectRows[ARow] := 'Y'
  else
    fSelectRows[ARow] := 'N';
  Invalidate;
end;

procedure TSortGrid.DeSelectAll;
var
  I: Integer;
begin
  for I := 0 to fSelectRows.Count-1 do
    fSelectRows[I] := 'N';
end;

procedure TSortGrid.SetMultiSelect(value:Boolean);
begin
  if value<>FMultiSelect then
    FMultiSelect:=value;
end;

function TSortGrid.GetSelectRowCount:integer;
var
  idx,cnt:integer;
begin
  cnt:=0;
  for idx:=0 to FSelectRows.Count-1 do
  if FSelectRows[idx]='Y' then inc(Cnt);
  result:=cnt;
end;




{******TableEdit procedures****************************************************}

procedure TSortGrid.SizeChanged(OldColCount, OldRowCount: Longint);
begin
  inherited SizeChanged(OldColCount, OldRowCount);
  UpdateScrollPage;
  if fUpdateOnSizeChange = True then
    SortColumn(SortOptions.fSortCol);

  if Assigned(fOnSizeChanged) then
    fOnSizeChanged(Self, OldColCount, OldRowCount);
end;

//Check if Grid is modified
procedure TSortGrid.SetEditText(ACol, ARow: Longint; const Value: string);
begin
  try
    if Value <> Cells[ACol, ARow] then FModified:=True;
  finally
    inherited SetEditText(ACol, ARow, Value);
  end;
end;

procedure TSortGrid.BeginUpdate;
begin
  Screen.Cursor := crHourGlass;
  LockWindowUpdate(TForm(Parent).Handle);
  OldUpdateSize := fUpdateOnSizeChange;
  fUpdateOnSizeChange := False;
end;

procedure TSortGrid.EndUpdate;
begin
  LockWindowUpdate(0);
  fUpdateOnSizeChange := OldUpdateSize;
  Screen.Cursor := crDefault;
end;

//AutoSizes the ACol column.
procedure TSortGrid.AutoSizeCol(const ACol: LongInt);
var
  MaxWidth, TextW, HeaderWidth, i: Integer;
  FmtOpts: TFormatOptions;
begin
  //Resize the column to display the largest value.
  MaxWidth := 0;
  HeaderWidth := 0;
  for I := 0 to RowCount-1 do
  begin
    InitializeFormatOptions(Col,i,FmtOpts);
    Canvas.Font := FmtOpts.Font;
    TextW := Canvas.TextWidth(Cells[ACol, I]);

    if I = 0 then
      HeaderWidth := TextW
    else
      if TextW > MaxWidth then MaxWidth := TextW;
  end;

  if HeaderWidth > MaxWidth then
    MaxWidth := HeaderWidth;

  if ACol = SortOptions.fSortCol then
    Inc(MaxWidth, 12);

  if MaxWidth > 0 then
    ColWidths[ACol] := MaxWidth+Canvas.TextWidth('X')
  else
    ColWidths[ACol] := Canvas.TextWidth('xx');
end;

//AutoSizes ALL columns
procedure TSortGrid.AutoSizeColumns;
var
  iCol: Integer;
begin
  for iCol := 0 to ColCount-1 do
    AutoSizeCol(iCol);
end;

//Clears the grid.
procedure TSortGrid.Clear;
var
  I: Longint;
begin
  BeginUpdate;
  
  for I := 1 to (RowCount-1) do
    Rows[I].Clear;
  FModified := False;
  RowCount := 2;
  AutoSizeColumns;

  EndUpdate;
end;

//Jumps to row with sortgrid.cells[0, arow][1] = key
procedure TSortGrid.KeyPress(var Key: Char);
var
  iRow: Integer;
  Moved: Boolean;
begin
  if EditorMode = False then
  begin
    Moved := False;
    Key := UpCase(Key);
    for iRow := Row+1 to RowCount-1 do
    if (Length(Cells[0,iRow]) > 0) and (UpCase(Cells[0,iRow][1]) = Key) then
    begin
      MoveTo(0,iRow);
      Moved := True;
      Break;
    end;

    if Moved = False then
    for iRow := 1 to Row do
    if (Length(Cells[0,iRow]) > 0) and (UpCase(Cells[0,iRow][1]) = Key) then
    begin
      MoveTo(0,iRow);
      Break;
    end;
  end;

  inherited KeyPress(Key);
end;

//Moves the selected cell to (ACol, ARow) and makes it visible.
procedure TSortGrid.MoveTo(ACol, ARow: Longint);
var
  MaxRow: Integer;
begin
  if ACol < 0 then ACol:=0;
  if ARow < 1 then ARow := 1;

  if SelectCell(ACol, ARow) = True then
  begin
    Col := ACol;
    Row := ARow;
    MoveColRow(ACol, ARow, True, True);
    LockWindowUpdate(TForm(Parent).Handle);

    //Make sure the selected Coloumn is Visible
    if (ACol > (LeftCol+VisibleColCount-1)) or (ACol < LeftCol) then
      repeat
        LeftCol := LeftCol+1;
      until
        (ACol <= (LeftCol+VisibleColCount-1)) and (ACol >= LeftCol);

    //Make sure the selected Row is at the top
    MaxRow := RowCount-VisibleRowCount;
    if MaxRow < 1 then MaxRow := 1;

    if ARow > MaxRow then
      TopRow := MaxRow
    else
    begin
      if ARow < 2 then
        TopRow := 1
      else
        TopRow := ARow-1;
    end;
    LockWindowUpdate(0);
  end;
end;

procedure TSortGrid.InsertRow(ARow: Longint);
begin
     RowCount:=RowCount+1;
     MoveRow(RowCount-1, ARow);
     Rows[ARow].Clear;
     Row:=ARow;
     if Assigned(fOnRowInsert) then fOnRowInsert(Self, ARow);
end;

procedure TSortGrid.InsertColumn(ACol: Longint);
begin
     ColCount:=ColCount+1;
     MoveColumn(ColCount-1, ACol);
     Cols[ACol].Clear;
     Col:=ACol;
     if Assigned(fOnColumnInsert) then fOnColumnInsert(Self, ACol);
end;

procedure TSortGrid.DeleteRow(ARow: Longint);
var
  ASE: Boolean;
begin
  Rows[ARow].Clear;
  {If goAlwaysShowEditor is enabled then DeleteRow and MoveRow leave the caret past the last row or
    in one of the fixed rows.  So I turn it off before the delete and then back on after to get it working correctly.}
  if goAlwaysShowEditor in Options then
  begin
    Options:=Options-[goAlwaysShowEditor];
    ASE:=True;
  end
  else
    ASE:=False;

  inherited DeleteRow(ARow);

  if ASE then Options:=Options+[goAlwaysShowEditor];
  if Assigned(fOnRowDelete) then fOnRowDelete(Self, ARow);
end;

procedure TSortGrid.DeleteColumn(ACol: Longint);
var
  ASE: Boolean;
begin
  Cols[ACol].Clear;
  //See DeleteRow for comments...
  ASE:=False;
  if goAlwaysShowEditor in Options then
  begin
    Options:=Options-[goAlwaysShowEditor];
    ASE:=True;
  end;
  inherited DeleteColumn(ACol);
  if ASE then Options:=Options+[goAlwaysShowEditor];
  if Assigned(fOnColumnDelete) then fOnColumnDelete(Self, ACol);
end;

procedure TSortGrid.MoveRow(FromIndex, ToIndex: Longint);
var
  ASE: Boolean;
begin
  //See DeleteRow for comments...
  ASE:=False;
  if goAlwaysShowEditor in Options then
  begin
    Options:=Options-[goAlwaysShowEditor];
    ASE:=True;
  end;
  inherited MoveRow(FromIndex, ToIndex);
  if ASE then Options:=Options+[goAlwaysShowEditor];
end;

procedure TSortGrid.MoveColumn(FromIndex, ToIndex: Longint);
var
  ASE: Boolean;
begin
  //See DeleteRow for comments...
  ASE:=False;
  if goAlwaysShowEditor in Options then
  begin
    Options:=Options-[goAlwaysShowEditor];
    ASE:=True;
  end;
  inherited MoveColumn(FromIndex, ToIndex);
  if ASE then Options:=Options+[goAlwaysShowEditor];
end;

//The logic gets around a weird case where you swap with the last row.
procedure TSortGrid.SwapRows(ARow1, ARow2: Longint);
begin
     if ARow1 < ARow2 then
     begin
          MoveRow(ARow2, ARow1);
          MoveRow(ARow1+1, ARow2);
     end
     else if ARow2 < ARow1 then
     begin
          MoveRow(ARow1, ARow2);
          MoveRow(ARow2+1, ARow1);
     end;
end;

//The logic gets around a weird case where you swap with the last column.
procedure TSortGrid.SwapColumns(ACol1, ACol2: Longint);
begin
     if ACol1 < ACol2 then
     begin
          MoveColumn(ACol2, ACol1);
          MoveColumn(ACol1+1, ACol2);
     end
     else if ACol2 < ACol1 then
     begin
          MoveColumn(ACol1, ACol2);
          MoveColumn(ACol2+1, ACol1);
     end;
end;




{*******Sort*******************************************************************}

procedure TSortGrid.SetSortOptions(value: TSortOptions);
begin
 fSortOptions.Assign(value);
end;

function TSortedList.GetItem(const i: Integer): PSortedListEntry;
begin
  //Cast the pointer.
  Result:=PSortedListEntry(Items[i]);
end;

procedure TSortedList.Reset;
var
  I: Integer;
begin
     //Dispose of anything in the list first.
     for i:=0 to Count-1 do
         if Items[i] <> nil then
            Dispose(Items[i]);
     //Now clear the list.
     Clear;
end;

function TSortGrid.DetermineSortStyle(const ACol: Longint): TSortStyle;
var
  i: Integer;
  DoNumeric, DoDateTime, DoTime: Boolean;
begin
  DoNumeric:=True;
  DoDateTime:=True;
  DoTime:=True;

  //Note: We only go through the rows once.
  //This code depends on the fact that no
  //entry can be both a date and number.
  for i:=1 to RowCount-1 do
  begin
    if DoNumeric then
    begin
      try
        StrToFloat(Cells[ACol, i]);
      except
        on EConvertError do DoNumeric:=False;
      end;
    end;

    if DoTime then
    begin
      try
        StrToTime(Cells[ACol, i]);
      except
        on EConvertError do DoTime:=False;
      end;
    end;

    if DoDateTime then
    begin
      try
        StrToDateTime(Cells[ACol, i]);
      except
        on EConvertError do DoDateTime:=False;
      end;
    end;
  end;

  if DoNumeric then Result := ssNumeric
  else if DoDateTime then Result := ssDateTime
  else if DoTime then Result := ssTime
  else Result := ssNormal;
end;

function ExtendedCompare(const Str1, Str2: String): Integer;
var
  S,S2: String;
  I: Integer;
begin
  S := Str1;
  S2 := Str2;

  if (S = '') or (S2 = '') then
  begin
    Result := -1;
    Exit;
  end;

  if S[1] in ['0','1','2','3','4','5','6','7','8','9'] then
    S[1] := S[1]  //just do something
  else
    for I := 1 to Length(S) do
    begin
      if S[I] <> S2[I] then
      begin
        Delete(S,1,I-1);
        Delete(S2,1,I-1);
        Break;
      end;
    end;

  if S = S2 then
    Result := 0
  else
    try
      StrToInt(S);
      StrToInt(S2);

      for I := 1 to Length(S) do
      if S[I] in ['0','1','2','3','4','5','6','7','8','9'] then
        S := Copy(S,1,I+1);

      for I := 1 to Length(S2) do
      if S2[I] in ['0','1','2','3','4','5','6','7','8','9'] then
        S2 := Copy(S2,1,I+1);

      if StrToInt(S) > StrToInt(S2) then
        Result := 1
      else
        Result := -1;
    except
      Result := StrComp(PChar(S), PChar(S2));
    end
end;

function NormalCompare(const Str1, Str2: String): Integer;
begin
  Result := StrComp(PChar(Str1), PChar(Str2));
end;

function DateTimeCompare(const Str1, Str2: String): Integer;
var
  Val1, Val2: TDateTime;
begin
  try
    Val1:=StrToDateTime(Str1);
    Val2:=StrToDateTime(Str2);
    if Val1 < Val2 then Result:=-1
    else if Val2 < Val1 then Result:=1
    else Result:=0;
  except
    on EConvertError do Result:=0;
  end;
end;

function TimeCompare(const Str1, Str2: String): Integer;
var
  Val1, Val2: TDateTime;
begin
  try
    Val1:=StrToTime(Str1);
    Val2:=StrToTime(Str2);
    if Val1 < Val2 then Result:=-1
    else if Val2 < Val1 then Result:=1
    else Result:=0;
   except
     on EConvertError do Result:=0;
   end;
end;

function NumericCompare(const Str1, Str2: String): Integer;
var
  Val1, Val2: Extended;
begin
  try
    Val1:=StrToFloat(Str1);
    Val2:=StrToFloat(Str2);
    if Val1 < Val2 then Result:=-1
    else if Val2 < Val1 then Result:=1
    else Result:=0;
  except
    on EConvertError do Result:=0;
  end;
end;

function Compare(Item1, Item2: Pointer): Integer;
var
  Entry1, Entry2: PSortedListEntry;
begin
  Entry1 := Item1;
  Entry2 := Item2;

  //Handle Case-Insensitivity.
  if VarSortOptions.SortCaseSensitive = False then
  begin
    Entry1^.Str := Lowercase(Entry1^.Str);
    Entry2^.Str := Lowercase(Entry2^.Str);
  end;

  //Determine compare type and do the comparison.
  case VarSortOptions.SortStyle of
//      ssExtended: Result:=ExtendedCompare(Entry1^.Str, Entry2^.Str);
      ssNumeric: Result:=NumericCompare(Entry1^.Str, Entry2^.Str);
      ssDateTime: Result:=DateTimeCompare(Entry1^.Str, Entry2^.Str);
      ssTime: Result:=TimeCompare(Entry1^.Str, Entry2^.Str);
    else
      Result:=NormalCompare(Entry1^.Str, Entry2^.Str);
  end;

  //Now, make sure we don't swap the rows if the Keys are equal.
  //If they're equal then we sort by row number.
  if Result = 0 then
  begin
    if Entry1^.RowNum < Entry2^.RowNum then Result:=-1
    else if Entry1^.RowNum > Entry2^.RowNum then Result:=1
    else Result:=0; //Sometimes an item does get compared to itself.
  end
    else //Reverse polarity if descending sort.
    if VarSortOptions.SortDirection = sdDescending then
      Result:=-1*Result;
end;

//Sorts the variable rows using Column ACol as a key
procedure TSortGrid.SortColumn(ACol: LongInt);
var
  I,I2,iCol: Integer;
  Item: PSortedListEntry;
  BufferGrid: TStringGrid;
  StringList: TStringList;
begin
  if Assigned(fOnBeginSort) then fOnBeginSort(Self, ACol);
  //Let everyone know we're sorting.
  fSorting:=True;

  BeginUpdate;
  SortOptions.fSortCol := ACol;

  StringList := TStringList.Create;
  StringList.CommaText := fSecondaryIndex;
  if StringList.IndexOf(IntToStr(ACol)) > -1 then
    StringList.Delete( StringList.IndexOf(IntToStr(ACol)) );//don't sort col twice
  StringList.Insert(0, IntToStr(ACol));

  for I2 := StringList.Count-1 downto 0 do//Loop to sort all columns
  begin
    iCol := StrToInt(StringList[I2]);
    if RowCount > 2 then
    begin
      if SortOptions.SortStyle = ssAutomatic then
        SortOptions.SortStyle := DetermineSortStyle(iCol);

     try
        fSortedList.Reset;
        //Insert the Row Number and Key (Str) into
        for I:=1 to RowCount-1 do
        begin
          New(Item);
          Item^.RowNum := I;
          Item^.Str:=Cells[iCol, I];
          fSortedList.Add(Item);
        end;

        VarSortOptions := SortOptions;
        fSortedList.Sort(Compare);

        //Now rearrange the rows of the grid in sorted order.
        //This is a fast but space inefficient way to do it.
        //First, create a buffer grid and size it correctly.
        BufferGrid:=TStringGrid.Create(Self);
        try
          BufferGrid.ColCount:=ColCount;
          BufferGrid.RowCount:=RowCount;
          //Copy the rows to the buffer grid in sorted order.
          for i:=0 to fSortedList.Count-1 do
          begin
            Item:=fSortedList.GetItem(i);
            BufferGrid.Rows[i+1].Assign(Rows[Item^.RowNum]);
          end;
          //Now put the rows back into the original grid.
          for I:=1 to RowCount-1 do
            Rows[I].Assign(BufferGrid.Rows[I]);
        finally
          BufferGrid.Free;
        end;
        //Now put the selection back on the right row.
        for i:=0 to fSortedList.Count-1 do
        begin
          Item:=fSortedList.GetItem(i);
          if Item^.RowNum = Row then
          begin
//MoveTo(Col, i+1);
            Break;
          end;
        end;
      finally
        FSorting := False;
        MoveTo(0,Row);
      end;

      if Assigned(fOnEndSort) then fOnEndSort(Self, iCol);
    end;
  end;

  MoveTo(ACol,Row);

  StringList.Free;
  EndUpdate;
end;





{*******Search & Filter********************************************************}

procedure TSearchOptions.SetGrid(Grid: TSortGrid );
begin
  fGrid := Grid;
end;

procedure TSortGrid.SetSearchOptions(value: TSearchOptions);
begin
 fSearchOptions.Assign(value);
end;

procedure TSearchOptions.SetFiltered(Value: Boolean);
begin
  fFiltered := Value;
  if Assigned(FGrid) then
  begin
    FGrid.BeginUpdate;

    if Filtered = True then
    begin
      FilterCount := 0;
      FGrid.Search(1);
    end
    else
    begin
      if FilterCount < 0 then
      begin
        FGrid.RowCount := FGrid.RowCount+(-1*FilterCount);
        FGrid.DeleteRow(1);
      end
      else
        FGrid.RowCount := FGrid.RowCount+FilterCount;
      FilterCount := 0;
    end;
    FGrid.SortColumn( FGrid.SortOptions.fSortCol );
    FGrid.EndUpdate;
  end;
end;

function TSortGrid.FindFirst: Boolean;
begin
  Result := Search(1);
end;

function TSortGrid.FindNext: Boolean;
begin
  Result := Search(Selection.Bottom+1);
end;

function TSortGrid.Search(StartRow: Cardinal): Boolean;
var
  iRow, iCol: Cardinal;
  SearchStr, CellStr: String;
begin
  Result := False;
  if SearchOptions.CaseSensitive = False then
    SearchStr := LowerCase(SearchOptions.SearchText)
  else
    SearchStr := SearchOptions.SearchText;

  if SearchOptions.SearchCol > -1 then//Only search one col
  for iRow := StartRow to RowCount-1 do
  begin
    CellStr := Cells[SearchOptions.SearchCol, iRow];
    if SearchOptions.CaseSensitive = False then
      CellStr := LowerCase(CellStr);
    if Pos(SearchStr, CellStr) > 0 then
    begin
      if SearchOptions.Filtered = True then
      begin
        MoveRow(iRow, 1);
        Inc(FilterCount);
        Result := True;
      end
      else
      begin
        MoveTo(SearchOptions.SearchCol, iRow);
        Result := True;
        Break;
      end;
    end;
  end
  else //Search all Cells
  for iRow := StartRow to RowCount-1 do
  begin
    if (Result = True) and (SearchOptions.Filtered = False) then
      Break;
    for iCol := 0 to ColCount-1 do
    begin
      CellStr := Cells[iCol, iRow];
      if SearchOptions.CaseSensitive = False then
        CellStr := LowerCase(CellStr);
      if Pos(SearchStr, CellStr) > 0 then
      begin
        if SearchOptions.Filtered = True then
        begin
          MoveRow(iRow, 1);
          Inc(FilterCount);
          Result := True;
          Break;
        end
        else
        begin
          MoveTo(iCol, iRow);
          Result := True;
          Break;
        end;
      end;
    end;
  end;

  if SearchOptions.Filtered = True then
  begin
    MoveTo(0,1);
    fUpdateOnSizeChange := True;
    if FilterCount = 0 then
    begin //Because FixedRowCount < RowCount
      FilterCount := (-1*RowCount)+1;
      SwapRows(1, RowCount);
      RowCount := 2;
    end
    else
    if FilterCount = 1 then
    begin
      FilterCount := RowCount-2;
      RowCount := 2;
    end
    else
    if FilterCount > 1 then
    begin
      FilterCount := RowCount-FilterCount-1;
      RowCount := RowCount-FilterCount;
    end;
  end;
end;





{*******File*******************************************************************}

procedure TSortGrid.SaveToFile(const FileName: String; SaveWidth: Boolean);
var
  List1, List2: TStringList;
  I: Integer;
begin
  BeginUpdate;
  SearchOptions.Filtered := False;
  List1 := TStringList.Create;
  List2 := TStringList.Create;
  //Save Windows size
  if SaveWidth = True then
  begin
    List1.Add( IntToStr( TForm(Parent).Width) );//sortgrid pos
    List1.Add( IntToStr( TForm(Parent).Height) );
    for I := 0 to ColCount-1 do  //col width
      List1.Add( IntToStr(ColWidths[I]) );
    List1.Add( IntToStr( TForm(Parent).Left) ); //parent form pos
    List1.Add( IntToStr( TForm(Parent).Top) );
  end
  else
    for I := 0 to ColCount+3 do
      List1.Add( '-1' );
  List2.Add( List1.CommaText );

  for I := 0 to RowCount-1 do
    List2.Add( Rows[I].CommaText );

  List2.SaveToFile( FileName );
  List1.Free;
  List2.Free;
  EndUpdate;
end;

function TSortGrid.LoadFromFile(const FileName: String; LoadWidth: Boolean): Boolean;
var
  List1, List2: TStringList;
  iCol, iRow: Integer;
begin
  Result := False;
  List1 := TStringList.Create;
  List2 := TStringList.Create;

  try
    List1.LoadFromFile( FileName );
    //Validate file
    List2.CommaText := List1[0];
    for iCol := 0 to List2.Count-1 do
      StrToint(List2[iCol]);
    //File Header is OK, so the rest is probaly also OK
    Clear;
    BeginUpdate;
    //Load Contents
    for iRow := 1 to List1.Count-1 do
    begin
      List2.CommaText := List1[iRow];
      for iCol := 0 to List2.Count-1 do
        Cells[iCol,iRow-1] := List2[iCol];
    end;
    RowCount := List1.Count-1;
    ColCount := List2.Count;

    //Restore saved Size
    if LoadWidth = True then
    begin
      List2.CommaText := List1[0];
      //sortgrid pos
      if List2[0] <> '-1' then
        TForm(Parent).Width := StrToInt(List2[0]);
      if List2[1] <> '-1' then
        TForm(Parent).Height := StrToInt(List2[1]);
      //col width
      for iCol := 0 to ColCount-1 do
        if List2[iCol+2] = '-1' then
          AutoSizeCol(iCol)
        else
          ColWidths[iCol] := StrToInt(List2[iCol+2]);
      //parent form pos. new in 1.30+
      try
        if List2[ColCount+2] <> '-1' then
          TForm(Parent).Left := StrToInt(List2[ColCount+2]);
        if List2[ColCount+3] <> '-1' then
          TForm(Parent).Top := StrToInt(List2[ColCount+3]);
      except
      end;
    end
    else
      AutoSizeColumns;
    //Sort properly
    SortOptions.fSortDirection := sdAscending;
    SortColumn(0);

    fModified := False;
    Result := True;
    EndUpdate;
  except
    Result := False;
    EndUpdate;
  end;

  List1.Free;
  List2.Free;
end;

function TSortGrid.PageCount: Integer;
begin
  fPageCount := 0;
  DrawToCanvas(nil, pmPageCount, 1,RowCount-1);
  result := fPageCount;
end;

procedure TSortGrid.PrintPreview(Image: TImage);
begin
  fPageCount := 0;
  DrawToCanvas(Image.Canvas, pmPreview, 1,RowCount-1);
end;

procedure TSortGrid.Print;
begin
  if Printer.Printers.Count = 0 then
  begin
    MessageDlg('No Printer is installed', mtError, [mbOK],0);
    Exit;
  end;
  Printer.Title := PrintOptions.fJobTitle;
  Printer.Copies := PrintOptions.fCopies;
  Printer.BeginDoc;
  DrawToCanvas(Printer.Canvas, pmPrint, PrintOptions.FromRow,PrintOptions.ToRow);
  Printer.EndDoc;
end;

procedure TSortGrid.DrawToCanvas(ACanvas: TCanvas; Mode: TPrintMode; FromRow, ToRow: Integer);
var
  PageWidth, PageHeight, PageRow,PageCol,I, iRow, FromCol,ToCol, X,Y: Integer;
  DoPaint: Boolean;

  function ScaleX(I:Integer): Integer;
  begin
    if Mode = pmPreview then
      Result := I
    else
      Result := I * (GetDeviceCaps(Printer.Handle, LOGPIXELSX) div Screen.PixelsPerInch);
  end;
  function ScaleY(I:Integer): Integer;
  begin
    if Mode = pmPreview then
      Result := I
    else
      Result := I * (GetDeviceCaps(Printer.Handle, LOGPIXELSY) div Screen.PixelsPerInch);
  end;

  procedure DrawCells(iRow:Integer);
  var
    iCol,I: Integer;
    R: TRect;
  begin
//Alignment must be done another day
    for iCol := FromCol to ToCol do
    begin
      //X Offset
      X := 0;
      for I := FromCol to iCol-1 do
        Inc(X, ScaleX(ColWidths[I]+1));
      //Text Rect
      R := Rect(X,Y, X+ScaleX(ColWidths[iCol]), Y+ScaleY(RowHeights[iRow]));
      //Draw on the Canvas
      if DoPaint then
        ACanvas.TextRect(R, X, Y, Cells[iCol, iRow]);
    end;
  end;

  procedure DrawTitle; //draw Header and Footer
  var
    S: String;
tmpfont:tfont;//I have no idea why you can't use gettextwidth when acanvas = printer.canvas, it returns wrong value
  begin
    if DoPaint then
begin
      ACanvas.Font.Size := 10;
tmpfont:=font;
canvas.font := acanvas.font;
end;
    //Title
    S := PrintOptions.PageTitle;
    if DoPaint then
      ACanvas.TextOut( (PageWidth div 2) - (ScaleX(Canvas.TextWidth(S) div 2)), ScaleY(1), S);
    //Page nr
    S := 'Page '+IntToStr(PageRow);
    if (ToCol < ColCount-1) or (PageCol > 1) then
      S := S+'-'+IntToStr(PageCol);
    if DoPaint then
      ACanvas.TextOut( PageWidth - ScaleX(Canvas.TextWidth(S)+10), PageHeight-ScaleY(19), S);

    if DoPaint then
begin
      ACanvas.Font.Size := Font.Size;
canvas.font := tmpfont;//Delphi 4.0 warning is wrong
end;
    Y := ScaleY(20);
    DrawCells(0);
  end;

begin
  //page size
  PageWidth := Printer.PageWidth;
  PageHeight := Printer.PageHeight;
  if Mode = pmPreview then
  begin
    PageWidth := PageWidth div ((GetDeviceCaps(Printer.Handle, LOGPIXELSX) div Screen.PixelsPerInch));
    PageHeight := PageHeight div ((GetDeviceCaps(Printer.Handle, LOGPIXELSY) div Screen.PixelsPerInch));
    ACanvas.Brush.Color := ClWhite;
    ACanvas.FillRect( Rect(0,0,PageWidth,PageHeight));
  end;
  if Mode <> pmPageCount then
  begin
    ACanvas.Font := Font;
    ACanvas.Font.Color := clBlack;
  end;
  PageCol := 0;
  FromCol := -2;
  ToCol := -1;
  //scan cols
  repeat
    //Scan missing cols
    if FromCol = ToCol then
      Inc(FromCol)
    else
      FromCol := ToCol+1;
    Inc(ToCol);
    //Get Cols with width that fits page
    X := 0;
    for I := FromCol to ColCount-1 do
    begin
      Inc(X, ScaleX(ColWidths[I]+1));
      if X <= PageWidth then
        ToCol := I;
    end;
    PageRow := 1;
    Inc(PageCol);
    //Mode = PageCount
    Inc(fPageCount);
    //preview mode
    DoPaint := (((Mode = pmPreview) and (fPageCount = PrintOptions.PreviewPage)) or (Mode = pmPrint));
    //Header & Footer
    DrawTitle;
    //Contents
    iRow := FromRow;
    repeat
      Inc(Y, ScaleY(RowHeights[iRow]));
      if Y <= PageHeight-ScaleY(20) then
      begin //draw contents to canvas
        DrawCells(iRow);
        Inc(iRow);
      end
      else//New page
      begin
        if (DoPaint = True) and (Mode = pmPreview) then
          Exit;
        if Mode = pmPrint then
          Printer.NewPage;
        Inc(fPageCount);//pagecount
        DoPaint := (((Mode = pmPreview) and (fPageCount = PrintOptions.PreviewPage)) or (Mode = pmPrint));        
        Inc(PageRow);
        DrawTitle;
      end;
      if (iRow = ToRow+1) and (ToCol < ColCount-1) and (Y <= PageHeight-ScaleY(20)) then
      begin
        if (DoPaint = True) and (Mode = pmPreview) then
          Exit;
        if Mode = pmPrint then
          Printer.NewPage;
        DrawTitle;
      end;
    until
      iRow = ToRow+1;
  until
    ToCol = ColCount-1;
end;

end.
